Package: celda
Title: CEllular Latent Dirichlet Allocation
Version: 1.29.0
Authors@R: c(person("Joshua", "Campbell", email = "camp@bu.edu",
    role = c("aut", "cre")),
    person("Shiyi", "Yang", email="syyang@bu.edu", role = c("aut")),
    person("Zhe", "Wang", email="zhe@bu.edu", role = c("aut")),
    person("Sean", "Corbett", email = "scorbett@bu.edu", role = c("aut")),
    person("Yusuke", "Koga", email="ykoga07@bu.edu", role = c("aut")))
Description: Celda is a suite of Bayesian hierarchical models for
    clustering single-cell RNA-sequencing (scRNA-seq) data. It is able to
    perform "bi-clustering" and simultaneously cluster genes into gene modules
    and cells into cell subpopulations. It also contains DecontX, a novel
    Bayesian method to computationally estimate and remove RNA contamination in
    individual cells without empty droplet information. A variety of scRNA-seq
    data visualization functions is also included.
Depends: R (>= 4.0), SingleCellExperiment, Matrix
VignetteBuilder: knitr
Imports: plyr, foreach, ggplot2, RColorBrewer, grid, scales, gtable,
        grDevices, graphics, matrixStats, doParallel, digest, methods,
        reshape2, S4Vectors, data.table, Rcpp, RcppEigen, uwot,
        enrichR, SummarizedExperiment, MCMCprecision, ggrepel, Rtsne,
        withr, scater (>= 1.14.4), scran, dbscan, DelayedArray,
        stringr, ComplexHeatmap, gridExtra, circlize, dendextend,
        ggdendro, pROC
Suggests: testthat, knitr, roxygen2, rmarkdown, biomaRt, covr,
        BiocManager, BiocStyle, TENxPBMCData, singleCellTK,
        M3DExampleData
LinkingTo: Rcpp, RcppEigen
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
BugReports: https://github.com/campbio/celda/issues
biocViews: SingleCell, GeneExpression, Clustering, Sequencing,
        Bayesian, ImmunoOncology, DataImport
NeedsCompilation: yes
git_url: https://git.bioconductor.org/packages/celda
git_branch: devel
git_last_commit: ab63841
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
Packaged: 2026-04-28 21:46:39 UTC; biocbuild
Author: Joshua Campbell [aut, cre],
  Shiyi Yang [aut],
  Zhe Wang [aut],
  Sean Corbett [aut],
  Yusuke Koga [aut]
Maintainer: Joshua Campbell <camp@bu.edu>
