# Generated by roxygen2: do not edit by hand

export("celdaClusters<-")
export("celdaModules<-")
export("decontXcounts<-")
export("sampleLabel<-")
export(appendCeldaList)
export(availableModels)
export(bestLogLikelihood)
export(celda)
export(celdaClusters)
export(celdaGridSearch)
export(celdaHeatmap)
export(celdaModel)
export(celdaModules)
export(celdaPerplexity)
export(celdaProbabilityMap)
export(celdaTsne)
export(celdaUmap)
export(celda_C)
export(celda_CG)
export(celda_G)
export(celdatosce)
export(clusterProbability)
export(compareCountMatrix)
export(countChecksum)
export(decontX)
export(decontXcounts)
export(distinctColors)
export(factorizeMatrix)
export(featureModuleLookup)
export(featureModuleTable)
export(findMarkersTree)
export(geneSetEnrich)
export(logLikelihood)
export(logLikelihoodHistory)
export(matrixNames)
export(moduleHeatmap)
export(normalizeCounts)
export(params)
export(perplexity)
export(plotCeldaViolin)
export(plotDecontXContamination)
export(plotDecontXMarkerExpression)
export(plotDecontXMarkerPercentage)
export(plotDendro)
export(plotDimReduceCluster)
export(plotDimReduceFeature)
export(plotDimReduceGrid)
export(plotDimReduceModule)
export(plotGridSearchPerplexity)
export(plotHeatmap)
export(plotMarkerHeatmap)
export(plotRPC)
export(recodeClusterY)
export(recodeClusterZ)
export(recursiveSplitCell)
export(recursiveSplitModule)
export(reorderCelda)
export(reportCeldaCGPlotResults)
export(reportCeldaCGRun)
export(resList)
export(resamplePerplexity)
export(retrieveFeatureIndex)
export(runParams)
export(sampleLabel)
export(selectBestModel)
export(selectFeatures)
export(simulateCells)
export(simulateContamination)
export(splitModule)
export(subsetCeldaList)
export(topRank)
exportMethods("celdaClusters<-")
exportMethods("celdaModules<-")
exportMethods("decontXcounts<-")
exportMethods("sampleLabel<-")
exportMethods(bestLogLikelihood)
exportMethods(celdaClusters)
exportMethods(celdaGridSearch)
exportMethods(celdaHeatmap)
exportMethods(celdaModel)
exportMethods(celdaModules)
exportMethods(celdaPerplexity)
exportMethods(celdaProbabilityMap)
exportMethods(celdaTsne)
exportMethods(celdaUmap)
exportMethods(celda_C)
exportMethods(celda_CG)
exportMethods(celda_G)
exportMethods(celdatosce)
exportMethods(clusterProbability)
exportMethods(compareCountMatrix)
exportMethods(countChecksum)
exportMethods(decontX)
exportMethods(decontXcounts)
exportMethods(factorizeMatrix)
exportMethods(featureModuleLookup)
exportMethods(geneSetEnrich)
exportMethods(logLikelihood)
exportMethods(logLikelihoodHistory)
exportMethods(matrixNames)
exportMethods(moduleHeatmap)
exportMethods(params)
exportMethods(perplexity)
exportMethods(plotCeldaViolin)
exportMethods(plotDimReduceCluster)
exportMethods(plotDimReduceFeature)
exportMethods(plotDimReduceGrid)
exportMethods(plotDimReduceModule)
exportMethods(plotGridSearchPerplexity)
exportMethods(plotRPC)
exportMethods(recursiveSplitCell)
exportMethods(recursiveSplitModule)
exportMethods(reorderCelda)
exportMethods(resList)
exportMethods(resamplePerplexity)
exportMethods(runParams)
exportMethods(sampleLabel)
exportMethods(selectBestModel)
exportMethods(selectFeatures)
exportMethods(splitModule)
exportMethods(subsetCeldaList)
import(Rcpp)
import(RcppEigen)
import(dbscan)
import(foreach)
import(ggplot2)
import(grDevices)
import(graphics)
import(grid)
import(pROC)
import(uwot)
import(withr)
importClassesFrom(Matrix,dgCMatrix)
importClassesFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(MCMCprecision,fit_dirichlet)
importFrom(Matrix,colSums)
importFrom(Matrix,rowSums)
importFrom(Matrix,t)
importFrom(RColorBrewer,brewer.pal)
importFrom(Rtsne,Rtsne)
importFrom(data.table,as.data.table)
importFrom(dendextend,get_leaves_attr)
importFrom(dendextend,get_nodes_attr)
importFrom(dendextend,get_nodes_xy)
importFrom(digest,digest)
importFrom(doParallel,registerDoParallel)
importFrom(enrichR,enrichr)
importFrom(enrichR,listEnrichrDbs)
importFrom(ggdendro,dendro_data)
importFrom(ggdendro,ggdendrogram)
importFrom(ggrepel,geom_text_repel)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,colors)
importFrom(grDevices,hsv)
importFrom(grDevices,rgb2hsv)
importFrom(grid,grid.pretty)
importFrom(gtable,gtable)
importFrom(gtable,gtable_add_grob)
importFrom(gtable,gtable_height)
importFrom(gtable,gtable_width)
importFrom(matrixStats,logSumExp)
importFrom(methods,.hasSlot)
importFrom(methods,hasArg)
importFrom(methods,is)
importFrom(methods,new)
importFrom(pROC,auc)
importFrom(pROC,coords)
importFrom(pROC,roc)
importFrom(plyr,mapvalues)
importFrom(reshape2,melt)
importFrom(scales,brewer_pal)
importFrom(scales,dscale)
importFrom(scales,hue_pal)
importFrom(withr,with_seed)
importMethodsFrom(Matrix,"%*%")
useDynLib(celda,"_colSumByGroup")
useDynLib(celda,"_colSumByGroupChange")
useDynLib(celda,"_colSumByGroupChange_numeric")
useDynLib(celda,"_colSumByGroup_numeric")
useDynLib(celda,"_perplexityG")
useDynLib(celda,"_rowSumByGroup")
useDynLib(celda,"_rowSumByGroupChange")
useDynLib(celda,"_rowSumByGroupChange_numeric")
useDynLib(celda,"_rowSumByGroup_numeric")
