% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{celda}
\alias{celda}
\title{Celda models}
\usage{
celda()
}
\value{
None
}
\description{
List of available Celda models with correpsonding descriptions.
}
\examples{
celda()
}
