% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decon.R
\name{decontXcounts}
\alias{decontXcounts}
\alias{decontXcounts<-}
\alias{decontXcounts,SingleCellExperiment-method}
\alias{decontXcounts<-,SingleCellExperiment-method}
\title{Get or set decontaminated counts matrix}
\usage{
decontXcounts(object, ...)

decontXcounts(object, ...) <- value

\S4method{decontXcounts}{SingleCellExperiment}(object, ...)

\S4method{decontXcounts}{SingleCellExperiment}(object, ...) <- value
}
\arguments{
\item{object}{A \linkS4class{SingleCellExperiment} object.}

\item{...}{For the generic, further arguments to pass to each method.}

\item{value}{A matrix to save as an assay called \code{decontXcounts}}
}
\value{
If getting, the assay from \code{object} with the name
\code{decontXcounts} will be returned. If setting, a
\linkS4class{SingleCellExperiment} object will be returned with
\code{decontXcounts} listed in the \code{assay} slot.
}
\description{
Gets or sets the decontaminated counts matrix from a
a \linkS4class{SingleCellExperiment} object.
}
\seealso{
\code{\link{assay}} and \code{\link{assay<-}}
}
